@@ -31,9 +31,6 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import logging
-log = logging.getLogger("braille")
-
 import signal
 import os
 
@@ -71,8 +68,12 @@ from . import brltablenames
 from . import cmdnames
 from . import debug
 from . import eventsynthesizer
+from . import logger
 from . import orca_state
 
+_logger = logger.getLogger()
+log = _logger.newLog("braille")
+
 # Right now, the orca autogen.sh/configure needs a priori knowledge of
 # where the liblouis tables are.  When running autogen.sh/configure,
 # orca_platform.py:tablesdir will be set to point to the liblouis table
@@ -98,7 +99,7 @@ if louis and not tablesdir:
 #
 _monitor = None
 
-# brlapi keys which are not allowed to interupt speech:
+# brlapi keys which are not allowed to interrupt speech:
 #
 dontInteruptSpeechKeys = []
 if _brlAPIAvailable:
@@ -320,14 +321,9 @@ class Region:
           unreasonable amount of time (AKA Gecko).
         """
 
-        # Double check for ellipses.
-        #
-        maskSize = len(self.string) + (2 * self.string.count('\u2026'))
-
         # Create an empty mask.
         #
-        mask = ['\x00'] * maskSize
-        return "".join(mask)
+        return '\x00' * len(self.string)
 
     def repositionCursor(self):
         """Reposition the cursor offset for contracted mode.
@@ -527,8 +523,8 @@ class Text(Region):
         self.accessible = accessible
         if orca_state.activeScript and self.accessible:
             [string, self.caretOffset, self.lineOffset] = \
-                 orca_state.activeScript.getTextLineAtCaret(self.accessible,
-                                                            startOffset)
+                 orca_state.activeScript.getTextLineAtCaret(
+                     self.accessible, startOffset=startOffset, endOffset=endOffset)
         else:
             string = ""
             self.caretOffset = 0
@@ -537,7 +533,7 @@ class Text(Region):
         try:
             endOffset = endOffset - self.lineOffset
         except TypeError:
-            pass
+            endOffset = len(string)
 
         try:
             self.startOffset = startOffset - self.lineOffset
@@ -655,14 +651,14 @@ class Text(Region):
         #
         stringLength = len(self.rawLine) - len(self.label)
         lineEndOffset = self.lineOffset + stringLength
-        regionMask = [settings.TEXT_ATTR_BRAILLE_NONE]*stringLength
+        regionMask = [settings.BRAILLE_UNDERLINE_NONE]*stringLength
 
         attrIndicator = settings.textAttributesBrailleIndicator
         selIndicator = settings.brailleSelectorIndicator
         linkIndicator = settings.brailleLinkIndicator
         script = orca_state.activeScript
 
-        if getLinkMask and linkIndicator != settings.BRAILLE_LINK_NONE:
+        if getLinkMask and linkIndicator != settings.BRAILLE_UNDERLINE_NONE:
             try:
                 hyperText = self.accessible.queryHypertext()
                 nLinks = hyperText.getNLinks()
@@ -691,14 +687,14 @@ class Text(Region):
                                                     offset, True)
                 if endOffset <= offset:
                     break
-                mask = settings.TEXT_ATTR_BRAILLE_NONE
+                mask = settings.BRAILLE_UNDERLINE_NONE
                 offset = endOffset
                 for attrib in attributes:
                     if enabledAttributes.get(attrib, '') != '':
                         if enabledAttributes[attrib] != attributes[attrib]:
                             mask = attrIndicator
                             break
-                if mask != settings.TEXT_ATTR_BRAILLE_NONE:
+                if mask != settings.BRAILLE_UNDERLINE_NONE:
                     maskStart = max(startOffset - self.lineOffset, 0)
                     maskEnd = min(endOffset - self.lineOffset, stringLength)
                     for i in range(maskStart, maskEnd):
@@ -854,11 +850,7 @@ class Line:
             if region == _regionWithFocus:
                 focusOffset = len(string)
             if region.string:
-                # [[[TODO: WDW - HACK: Replace ellipses with "..."
-                # The ultimate solution is to get i18n support into
-                # BrlTTY.]]]
-                #
-                string += region.string.replace('\u2026', "...")
+                string += region.string
             mask = region.getAttributeMask(getLinkMask)
             attributeMask += mask
 
@@ -1061,7 +1053,7 @@ def _realignViewport(string, focusOffset, cursorOffset):
     # viewport.
     #
     if (cursorOffset < 0) \
-       or (settings.brailleAlignmentStyle == settings.ALIGN_BRAILLE_BY_EDGE) \
+       or (settings.brailleAlignmentStyle == settings.BRAILLE_ALIGN_BY_EDGE) \
        or not (cursorOffset >= viewport[0]
                and cursorOffset < (viewport[0] + _displaySize[0])):
         pass
@@ -1086,12 +1078,12 @@ def _realignViewport(string, focusOffset, cursorOffset):
         # in between the margins.  The only time we go outside the
         # margins are when we are at the ends of the string.
         #
-        if settings.brailleAlignmentStyle == settings.ALIGN_BRAILLE_BY_MARGIN:
+        if settings.brailleAlignmentStyle == settings.BRAILLE_ALIGN_BY_MARGIN:
             if cursorOffset < leftMargin:
                 jump = cursorOffset - leftMargin
             elif cursorOffset > rightMargin:
                 jump = cursorOffset - rightMargin
-        elif settings.brailleAlignmentStyle == settings.ALIGN_BRAILLE_BY_WORD:
+        elif settings.brailleAlignmentStyle == settings.BRAILLE_ALIGN_BY_WORD:
             # When we align by word, we want to try to show complete
             # words at the edges of the braille display.  When we're
             # near the left edge, we'll try to start a word at the
@@ -1149,6 +1141,7 @@ def _realignViewport(string, focusOffset, cursorOffset):
         #
         viewport[0] = min(viewport[0],
                           max(leftMostEdge, len(string) - _displaySize[0]))
+        viewport[0] = int(viewport[0])
 
 def refresh(panToCursor=True,
             targetCursorCell=0,
@@ -1279,7 +1272,7 @@ def refresh(panToCursor=True,
     #
     _realignViewport(string, focusOffset, cursorOffset)
 
-    startPos = viewport[0]
+    startPos = int(viewport[0])
     endPos = startPos + _displaySize[0]
 
     # Now normalize the cursor position to BrlTTY, which uses 1 as
@@ -1718,11 +1711,20 @@ def init(callback=None, tty=7):
                 "Braille module has been initialized using WINDOWPATH=" \
                 + "%s" % windowPath)
         except:
-            _brlAPI.enterTtyMode(tty)
-            _brlAPIRunning = True
-            debug.println(\
-                debug.LEVEL_CONFIGURATION,
-                "Braille module has been initialized using tty=%d" % tty)
+            try:
+                vtnr = os.environ["XDG_VTNR"]
+                _brlAPI.enterTtyModeWithPath()
+                _brlAPIRunning = True
+                debug.println(
+                    debug.LEVEL_CONFIGURATION,
+                    "Braille module has been initialized using XDG_VTNR=" \
+                    + "%s" % vtnr)
+            except:
+                _brlAPI.enterTtyMode(tty)
+                _brlAPIRunning = True
+                debug.println(
+                    debug.LEVEL_CONFIGURATION,
+                    "Braille module has been initialized using tty=%d" % tty)
         _brlAPISourceId = GLib.io_add_watch(_brlAPI.fileDescriptor,
                                             GLib.PRIORITY_DEFAULT,
                                             GLib.IO_IN,
