@@ -26,7 +26,9 @@ __copyright__ = "Copyright (c) 2013-2014 Igalia, S.L."
 __license__   = "LGPL"
 
 import pyatspi
+import time
 
+import orca.debug as debug
 import orca.orca as orca
 import orca.orca_state as orca_state
 import orca.scripts.default as default
@@ -55,25 +57,25 @@ class Script(default.Script):
             isComboBox = lambda x: x and x.getRole() == pyatspi.ROLE_COMBO_BOX
             newFocus = pyatspi.findAncestor(newFocus, isComboBox) or newFocus
             orca.setLocusOfFocus(event, newFocus, False)
+        elif self.utilities.isInOpenMenuBarMenu(newFocus):
+            window = self.utilities.topLevelObject(newFocus)
+            windowChanged = window and orca_state.activeWindow != window
+            if windowChanged:
+                orca_state.activeWindow = window
+                self.windowActivateTime = time.time()
 
         super().locusOfFocusChanged(event, oldFocus, newFocus)
 
     def onActiveDescendantChanged(self, event):
         """Callback for object:active-descendant-changed accessibility events."""
 
-        role = event.source.getRole()
+        if not self.utilities.isTypeahead(orca_state.locusOfFocus):
+            super().onActiveDescendantChanged(event)
+            return
 
-        try:
-            focusedRole = orca_state.locusOfFocus.getRole()
-        except:
-            pass
-        else:
-            # This is very likely typeahead search and not a real focus change.
-            tableRoles = [pyatspi.ROLE_TABLE, pyatspi.ROLE_TREE_TABLE]
-            if focusedRole == pyatspi.ROLE_TEXT and role in tableRoles:
-                orca.setLocusOfFocus(event, event.source, False)
-
-        default.Script.onActiveDescendantChanged(self, event)
+        msg = "GTK: locusOfFocus believed to be typeahead. Presenting change."
+        debug.println(debug.LEVEL_INFO, msg, True)
+        self.presentObject(event.any_data)
 
     def onCheckedChanged(self, event):
         """Callback for object:state-changed:checked accessibility events."""
@@ -110,88 +112,34 @@ class Script(default.Script):
 
         # NOTE: This event type is deprecated and Orca should no longer use it.
         # This callback remains just to handle bugs in applications and toolkits
-        # during the remainder of the unstable (3.11) development cycle.
+        # that fail to reliably emit object:state-changed:focused events.
 
-        role = event.source.getRole()
+        if self.utilities.eventIsCanvasNoise(event):
+            return
 
-        # https://bugzilla.gnome.org/show_bug.cgi?id=711397
-        if role == pyatspi.ROLE_COMBO_BOX:
+        if self.utilities.isLayoutOnly(event.source):
+            return
+
+        ancestor = pyatspi.findAncestor(orca_state.locusOfFocus, lambda x: x == event.source)
+        if not ancestor:
             orca.setLocusOfFocus(event, event.source)
             return
 
-        # The above issue also seems to happen with spin buttons.
-        if role == pyatspi.ROLE_SPIN_BUTTON:
-            orca.setLocusOfFocus(event, event.source)
+        isMenu = lambda x: x and x.getRole() == pyatspi.ROLE_MENU
+        if isMenu(ancestor) and not pyatspi.findAncestor(ancestor, isMenu):
             return
 
-        # And with sliders.
-        if role == pyatspi.ROLE_SLIDER:
-            orca.setLocusOfFocus(event, event.source)
+        orca.setLocusOfFocus(event, event.source)
+
+    def onFocusedChanged(self, event):
+        """Callback for object:state-changed:focused accessibility events."""
+
+        if self.utilities.isUselessPanel(event.source):
+            msg = "GTK: Event source believed to be useless panel"
+            debug.println(debug.LEVEL_INFO, msg, True)
             return
 
-        # https://bugzilla.gnome.org/show_bug.cgi?id=720987
-        if role == pyatspi.ROLE_TABLE_COLUMN_HEADER:
-            orca.setLocusOfFocus(event, event.source)
-            return
-
-        # https://bugzilla.gnome.org/show_bug.cgi?id=720989
-        if role == pyatspi.ROLE_MENU == event.source.parent.getRole():
-            orca.setLocusOfFocus(event, event.source)
-            return
-
-        # Unfiled, but a similar case of the above issue with combo boxes.
-        # Seems to happen for checkboxes too. This is why we can't have
-        # nice things.
-        if role in [pyatspi.ROLE_PUSH_BUTTON, pyatspi.ROLE_CHECK_BOX]:
-            orca.setLocusOfFocus(event, event.source)
-            return
-
-        # Unfiled. Happens in Evolution, but for what seems to be a generic
-        # Gtk+ toggle button. So we'll handle it here.
-        if role == pyatspi.ROLE_TOGGLE_BUTTON:
-            orca.setLocusOfFocus(event, event.source)
-            return
-
-        # Unfiled. But this happens when you are in Gedit, get into a menu
-        # and then press Escape. The text widget emits a focus: event, but
-        # not a state-changed:focused event.
-        #
-        # A similar issue can be seen when a text widget starts out having
-        # focus, such as in the old gnome-screensaver dialog.
-        if role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_PASSWORD_TEXT]:
-            orca.setLocusOfFocus(event, event.source)
-            return
-
-        # Unfiled. When a context menu first appears and an item is already
-        # selected, we get a focus: event for that menu item, but there is
-        # not a state-changed event for that item, nor a selection-changed
-        # event for the menu.
-        menuItems = [pyatspi.ROLE_CHECK_MENU_ITEM,
-                     pyatspi.ROLE_MENU_ITEM,
-                     pyatspi.ROLE_RADIO_MENU_ITEM]
-        if role in menuItems:
-            if orca_state.locusOfFocus \
-               and orca_state.locusOfFocus.parent != event.source.parent:
-                orca.setLocusOfFocus(event, event.source)
-            return
-
-        # Unfiled. When a canvas item gets focus but is not selected, we
-        # are only getting a focus event. This happens in Nautilus.
-        if role == pyatspi.ROLE_CANVAS and not self.utilities.eventIsCanvasNoise(event):
-            orca.setLocusOfFocus(event, event.source)
-            return
-
-        # Unfiled, but yet another case of only getting a focus: event when
-        # a widget appears in a parent container and is already focused.
-        # An example of this particular case is the list of elements dialogs.
-        if role == pyatspi.ROLE_TABLE:
-            obj = event.source
-            selectedChildren = self.utilities.selectedChildren(obj)
-            if selectedChildren:
-                obj = selectedChildren[0]
-
-            orca.setLocusOfFocus(event, obj)
-            return
+        super().onFocusedChanged(event)
 
     def onSelectedChanged(self, event):
         """Callback for object:state-changed:selected accessibility events."""
@@ -213,9 +161,19 @@ class Script(default.Script):
             super().onSelectionChanged(event)
             return
 
+        isFocused = event.source.getState().contains(pyatspi.STATE_FOCUSED)
         role = event.source.getRole()
-        if role == pyatspi.ROLE_COMBO_BOX \
-           and not event.source.getState().contains(pyatspi.STATE_FOCUSED):
+        if role == pyatspi.ROLE_COMBO_BOX and not isFocused:
+            return
+
+        if not isFocused and self.utilities.isTypeahead(orca_state.locusOfFocus):
+            msg = "GTK: locusOfFocus believed to be typeahead. Presenting change."
+            debug.println(debug.LEVEL_INFO, msg, True)
+
+            selectedChildren = self.utilities.selectedChildren(event.source)
+            for child in selectedChildren:
+                if not self.utilities.isLayoutOnly(child):
+                    self.presentObject(child)
             return
 
         if role == pyatspi.ROLE_LAYERED_PANE \
@@ -254,4 +212,7 @@ class Script(default.Script):
         if self.utilities.eventIsCanvasNoise(event):
             return False
 
+        if self.utilities.isUselessPanel(event.source):
+            return False
+
         return super().isActivatableEvent(event)
